#ifndef _sybdb_h_
#define _sybdb_h_

#include "tds.h"

#define FALSE 0
#define TRUE  1

#define DBSAVE   1
#define DBNOSAVE 0
#define DBNOERR  -1

#define INT_CANCEL 2

#define DBMAXNUMLEN 33
#define MAXNAME     30

#define DBVERSION_UNKNOWN 0
#define DBVERSION_46      1      
#define DBVERSION_100     2       

#define SYBAOPCNT  0x4b
#define SYBAOPCNTU 0x4c
#define SYBAOPSUM  0x4d
#define SYBAOPSUMU 0x4e
#define SYBAOPAVG  0x4f
#define SYBAOPAVGU 0x50
#define SYBAOPMIN  0x51
#define SYBAOPMAX  0x52

#define DBTXPLEN 16

typedef int RETCODE;
typedef int STATUS;

typedef struct {
void	*tds_login ;
} LOGINREC;

typedef struct {
void	*tds_socket ;
TDSRESULTINFO resinfo;
unsigned char dbbuf[4096];
} DBPROCESS;

typedef unsigned char DBBOOL ;
typedef unsigned char BOOL ;
typedef char DBCHAR ;
typedef unsigned char DBTINYINT ;
typedef short DBSMALLINT ;
typedef long DBINT	;
typedef char BYTE	;
typedef unsigned char DBBINARY ;
typedef float DBREAL ;
typedef double DBFLT8  ;
typedef struct dbnumeric
{
        BYTE         precision;
        BYTE         scale;
        BYTE         array[DBMAXNUMLEN];
} DBNUMERIC;
typedef struct dbmoney
{
        DBINT          mnyhigh;
        DBINT         mnylow;
} DBMONEY;
typedef struct dbmoney4
{
        DBINT          mny4;
} DBMONEY4;
typedef struct dbdatetime
{
	DBINT dtdays;
	DBINT dttime;
} DBDATETIME;
typedef struct dbdatetime4
{
	DBSMALLINT days;
	DBSMALLINT minutes;
} DBDATETIME4;

typedef struct dbdaterec
{
	DBINT	dateyear;
	DBINT	datemonth;
	DBINT	datedmonth;
	DBINT	datedyear;
	DBINT	datedweek;
	DBINT	datehour;
	DBINT	dateminute;
	DBINT	datesecond;
	DBINT	datemsecond;
	DBINT	datetzone;
} DBDATEREC;

typedef struct  dbtypeinfo
{
        DBINT   precision;
        DBINT   scale;
} DBTYPEINFO;
 

/* bind types */
#define CHARBIND          0
#define STRINGBIND        1
#define NTBSTRINGBIND     2
#define VARYCHARBIND      3
#define TINYBIND          6
#define SMALLBIND         7
#define INTBIND           8
#define FLT8BIND          9
#define REALBIND          10
#define DATETIMEBIND      11
#define SMALLDATETIMEBIND 12
#define MONEYBIND         13
#define SMALLMONEYBIND    14
#define BINARYBIND        15
#define BITBIND           16
#define NUMERICBIND       17
#define DECIMALBIND       18

#define DBPRCOLSEP  21
#define DBPRLINELEN 22
#define DBRPCRETURN 1

#define REG_ROW         -1
#define MORE_ROWS       -1
#define NO_MORE_ROWS    -2
#define BUF_FULL        -3
#define NO_MORE_RESULTS 2
#define SUCCEED         1
#define FAIL            0

#define DB_IN  1
#define DB_OUT 2
#define IN     1
#define OUT    2

#define DBSINGLE 0
#define DBDOUBLE 1
#define DBBOTH   2


RETCODE dbinit();
LOGINREC *dblogin();
RETCODE DBSETLPACKET(LOGINREC *login, short packet_size);
RETCODE DBSETLPWD(LOGINREC *login, char *password);
RETCODE DBSETLUSER(LOGINREC *login, char *username);
RETCODE DBSETLHOST(LOGINREC *login, char *hostname);
RETCODE DBSETLAPP(LOGINREC *login, char *application);
DBPROCESS *dbopen(LOGINREC *login,char *server);
DBINT dbconvert(DBPROCESS *dbproc, int srctype, BYTE *src, DBINT srclen, int desttype, BYTE *dest, DBINT destlen);
void dbexit();
RETCODE DBCMDROW(DBPROCESS *dbproc);
RETCODE dbsetdeflang(char *language);
int dbgetpacket(DBPROCESS *dbproc);
RETCODE dbsettime(int seconds);
RETCODE dbsetlogintime(int seconds);
DBINT dbretstatus(DBPROCESS *dbproc);
DBBOOL dbhasretstat(DBPROCESS *dbproc);
DBINT dbdatlen(DBPROCESS *dbproc, int column);
char *dbcolsource(DBPROCESS *dbproc,int colnum);
int dbcoltype(DBPROCESS *dbproc,int column);
DBINT DBCOUNT(DBPROCESS *dbproc);
int dbaltcolid(DBPROCESS *dbproc, int computeid, int column);
DBINT dbadlen(DBPROCESS *dbproc,int computeid, int column);
int dbalttype(DBPROCESS *dbproc, int computeid, int column);
BYTE *dbadata(DBPROCESS *dbproc, int computeid, int column);
int dbaltop(DBPROCESS *dbproc, int computeid, int column);
RETCODE dbsetopt(DBPROCESS *dbproc, int option, char *char_param, int int_param);
void dbsetinterrupt(DBPROCESS *dbproc, int (*ckintr)(),int (*hndlintr)());
RETCODE dbcancel(DBPROCESS *dbproc);
int dbnumrets(DBPROCESS *dbproc);
char *dbretname(DBPROCESS *dbproc, int retnum);
BYTE *dbretdata(DBPROCESS *dbproc, int retnum);
int dbretlen(DBPROCESS *dbproc, int retnum);
RETCODE dbsqlok(DBPROCESS *dbproc);
void dbprrow(DBPROCESS *dbproc);
void dbprhead(DBPROCESS *dbproc);
void dbloginfree(LOGINREC *login);
int dbnumalts(DBPROCESS *dbproc,int computeid);
BYTE *dbbylist(DBPROCESS *dbproc, int computeid, int size);
RETCODE dbrpcinit(DBPROCESS *dbproc,char *rpcname,DBSMALLINT options);
RETCODE dbrpcparam(DBPROCESS *dbproc, char *paramname, BYTE status, int type,
        DBINT maxlen, DBINT datalen, BYTE *value);
RETCODE dbrpcsend(DBPROCESS *dbproc);
RETCODE dbuse(DBPROCESS *dbproc,char *dbname);
DBBOOL DBDEAD(DBPROCESS *dbproc);
int (*dbmsghandle(handler))();
int (*dberrhandle(handler))();
RETCODE BCP_SETL(LOGINREC *login, DBBOOL enable);
RETCODE bcp_init(DBPROCESS *dbproc, char *tblname, char *hfile, char *errfile, int direction);
RETCODE bcp_collen(DBPROCESS *dbproc, DBINT varlen, int table_column);
RETCODE bcp_columns(DBPROCESS *dbproc, int host_colcount);
RETCODE bcp_colfmt(DBPROCESS *dbproc, int host_colnum, int host_type, int host_prefixlen, DBINT host_collen, BYTE *host_term,int host_termlen, int table_colnum);
RETCODE bcp_colfmt_ps(DBPROCESS *dbproc, int host_colnum, int host_type, int host_prefixlen, DBINT host_collen, BYTE *host_term,int host_termlen, int table_colnum, DBTYPEINFO *typeinfo);
RETCODE bcp_control(DBPROCESS *dbproc, int field, DBINT value);
RETCODE bcp_colptr(DBPROCESS *dbproc, BYTE *colptr, int table_column);
DBBOOL bcp_getl(LOGINREC *login);

RETCODE bcp_exec(DBPROCESS *dbproc, DBINT *rows_copied);
RETCODE bcp_readfmt(DBPROCESS *dbproc, char *filename);
RETCODE bcp_writefmt(DBPROCESS *dbproc, char *filename);
RETCODE bcp_sendrow(DBPROCESS *dbproc);
RETCODE bcp_moretext(DBPROCESS *dbproc, DBINT size, BYTE *text);
RETCODE bcp_batch(DBPROCESS *dbproc);
RETCODE bcp_done(DBPROCESS *dbproc);
RETCODE bcp_bind(DBPROCESS *dbproc, BYTE *varaddr, int prefixlen, DBINT varlen,
        BYTE *terminator, int termlen, int type, int table_column);
RETCODE dbmnyadd(DBPROCESS *dbproc, DBMONEY *m1, DBMONEY *m2, DBMONEY *sum);
RETCODE dbmnysub(DBPROCESS *dbproc, DBMONEY *m1, DBMONEY *m2, DBMONEY *diff);
RETCODE dbmnymul(DBPROCESS *dbproc, DBMONEY *m1, DBMONEY *m2, DBMONEY *prod);
RETCODE dbmnydivide(DBPROCESS *dbproc, DBMONEY *m1, DBMONEY *m2, DBMONEY *quotient);
RETCODE dbmnycmp(DBPROCESS *dbproc, DBMONEY *m1, DBMONEY *m2);
RETCODE dbmnyscale(DBPROCESS *dbproc, DBMONEY *dest,int multiplier, int addend);
RETCODE dbmnyzero(DBPROCESS *dbproc, DBMONEY *dest);
RETCODE dbmnymaxpos(DBPROCESS *dbproc, DBMONEY *dest);
RETCODE dbmnymaxneg(DBPROCESS *dbproc, DBMONEY *dest);
RETCODE dbmnyndigit(DBPROCESS *dbproc, DBMONEY *mnyptr,DBCHAR *value, DBBOOL *zero);
RETCODE dbmnyinit(DBPROCESS *dbproc,DBMONEY *mnyptr, int trim, DBBOOL *negative);
RETCODE dbmnydown(DBPROCESS *dbproc,DBMONEY *mnyptr, int divisor, int *remainder);
RETCODE dbmnyinc(DBPROCESS *dbproc,DBMONEY *mnyptr);
RETCODE dbmnydec(DBPROCESS *dbproc,DBMONEY *mnyptr);
RETCODE dbmnyminus(DBPROCESS *dbproc,DBMONEY *src, DBMONEY *dest);
RETCODE dbmny4cmp(DBPROCESS *dbproc, DBMONEY4 *m1, DBMONEY4 *m2);
RETCODE dbmny4minus(DBPROCESS *dbproc, DBMONEY4 *src, DBMONEY4 *dest);
RETCODE dbmny4zero(DBPROCESS *dbproc, DBMONEY4 *dest);
RETCODE dbmny4add(DBPROCESS *dbproc, DBMONEY4 *m1, DBMONEY4 *m2, DBMONEY4 *sum);
RETCODE dbmny4sub(DBPROCESS *dbproc, DBMONEY4 *m1, DBMONEY4 *m2, DBMONEY4 *diff);
RETCODE dbmny4mul(DBPROCESS *dbproc, DBMONEY4 *m1, DBMONEY4 *m2, DBMONEY4 *prod);
RETCODE dbmny4divide(DBPROCESS *dbproc, DBMONEY4 *m1, DBMONEY4 *m2, DBMONEY4 *quotient);
RETCODE dbmny4cmp(DBPROCESS *dbproc, DBMONEY4 *m1, DBMONEY4 *m2);
RETCODE dbdatecmp(DBPROCESS *dbproc, DBDATETIME *d1, DBDATETIME *d2);
RETCODE dbdatecrack(DBPROCESS *dbproc, DBDATEREC *dateinfo, DBDATETIME *datetime);
void dbrpwclr(LOGINREC *login);
RETCODE dbrpwset(LOGINREC *login, char *srvname, char *password, int pwlen);
void build_xact_string(char *xact_name, char *service_name, DBINT commid, char *result);
RETCODE remove_xact(DBPROCESS *connect, DBINT commid, int n);
RETCODE abort_xact(DBPROCESS *connect, DBINT commid);
void close_commit(DBPROCESS *connect);
RETCODE commit_xact(DBPROCESS *connect, DBINT commid);
DBPROCESS *open_commit(LOGINREC *login, char *servername);
RETCODE scan_xact(DBPROCESS *connect, DBINT commid);
DBINT start_xact(DBPROCESS *connect, char *application_name, char *xact_name, int site_count);
DBINT stat_xact(DBPROCESS *connect, DBINT commid);
int dbspid(DBPROCESS *dbproc);

#endif
