#ifndef _tds_h_
#define _tds_h_

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/signal.h>
#include <sys/wait.h>

#define TDS_CHAR char
#define TDS_TINYINT unsigned char
#define TDS_SMALLINT short
#define TDS_INT int

#define TDS_NO_MORE_ROWS     -2
#define TDS_SUCCEED          1
#define TDS_FAIL             0
#define TDS_NO_MORE_RESULTS  2
#define TDS_DONT_RETURN      42

#define TDS_RET_STAT_TOKEN  121
#define TDS_124_TOKEN       124  /* TDS 4.2 only */
#define TDS_COL_NAME_TOKEN  160  /* TDS 4.2 only */
#define TDS_COL_INFO_TOKEN  161  /* TDS 4.2 only */
#define TDS_169_TOKEN       169
#define TDS_ERR_TOKEN       170
#define TDS_MSG_TOKEN       171
#define TDS_LOGIN_ACK_TOKEN 173
#define TDS_174_TOKEN       174
#define TDS_ROW_TOKEN       209
#define TDS_226_TOKEN       226
#define TDS_ENV_CHG_TOKEN   227
#define TDS_229_TOKEN       229
#define TDS_RESULT_TOKEN    238
#define TDS_END_TOKEN       253
#define TDS_END2_TOKEN      254
#define TDS_255_TOKEN       255

/* states for tds_process_messages() */
#define PROCESS_ROWS    0
#define PROCESS_RESULTS 1
#define CANCEL          2
#define GOTO_1ST_ROW    3
#define LOGIN           4

/* 
<rant> Sybase does an awful job of this stuff, non null ints of size 1 2 
and 4 have there own codes but nullable ints are lumped into INTN
sheesh! </rant>
*/
#define SYBCHAR      47
#define SYBVARCHAR   39
#define SYBINTN      38
#define SYBINT1      48
#define SYBINT2      52
#define SYBINT4      56
#define SYBFLT8      62
#define SYBDATETIME  61
#define SYBBIT       50
#define SYBTEXT      35
#define SYBIMAGE     34
#define SYBMONEY4    112
#define SYBMONEY     60
#define SYBDATETIME4 58
#define SYBREAL      59
#define SYBBINARY    45
#define SYBVOID      31
#define SYBVARBINARY 37
#define SYBNUMERIC   108
#define SYBDECIMAL   106
#define SYBFLTN      109

/* FIX ME -- not a complete list */
#define is_fixed_type(x) (x==SYBINT1 || x==SYBINT2 || x==SYBINT4 || x==SYBFLT8 || x==SYBDATETIME || x==SYBBIT || x==SYBMONEY || x==SYBMONEY4)

typedef struct tds_login {
	TDS_CHAR  host_name[30];
	TDS_CHAR  user_name[30];
	TDS_CHAR  password[30];
	TDS_TINYINT bulk_copy; 
	TDS_CHAR  app_name[30];
	TDS_CHAR  server_name[30];
	TDS_TINYINT  major_version; /* TDS version */
	TDS_TINYINT  minor_version; /* TDS version */
	TDS_CHAR  library[11]; /* Ct-Library or DB-Library */
	TDS_CHAR language[30]; /* ie us-english */
	TDS_TINYINT encrypted; 
	TDS_CHAR char_set[30]; /*  ie iso_1 */
	TDS_SMALLINT block_size; 
} TDSLOGIN;

typedef struct tds_column_info {
	int column_type;
	int column_usertype;
	int column_size;
	char column_name[256];
	char column_value[256];
	int column_bindtype;
	int column_bindlen;
	long *column_nullbind;
	char *varaddr;
} TDSCOLINFO;

typedef struct tds_result_info {
	int rows_exist;
	long row_count;
	int num_cols;
	unsigned char more_results;
	TDSCOLINFO **columns;
} TDSRESULTINFO;

typedef struct tds_socket {
        int s;
	unsigned char *in_buf;
	unsigned char *out_buf;
	unsigned in_pos;
	unsigned out_pos;
	unsigned in_len;
	unsigned out_len;
	unsigned char out_flag;
	unsigned char last_packet;
	TDSRESULTINFO *res_info;
} TDSSOCKET;

int tds_write_packet(TDSSOCKET *tds,unsigned char final);
void tds_free_result_info(TDSRESULTINFO *info);
int tds_put_n(TDSSOCKET *tds, unsigned char *buf, int n);
int tds_put_string(TDSSOCKET *tds, unsigned char *buf, int n);
int tds_put_int(TDSSOCKET *tds, TDS_INT i);
int tds_put_smallint(TDSSOCKET *tds, TDS_SMALLINT si);
int tds_put_tinyint(TDSSOCKET *tds, TDS_TINYINT ti);
int tds_put_byte(TDSSOCKET *tds, unsigned char c);
unsigned char tds_get_byte(TDSSOCKET *tds);
void tds_unget_byte(TDSSOCKET *tds);
TDS_SMALLINT tds_get_smallint(TDSSOCKET *tds);
char *tds_get_n(TDSSOCKET *tds, void *dest, int n);
TDSRESULTINFO *tds_alloc_results(int num_cols);
#endif /* _tds_h_ */
